from numpy import(pi, cos, sin, tan, arccos, arctan, sqrt, deg2rad, rad2deg)

class StereogrphicProjection():
    def __init__(self, local_latitude, tropical_inclinaison=23.43811972, projection_radius=1.0):
        """
        Instantiate a stereogrphic projector.
 
        Args:
            projection_radius : The desired radius of the bigger tropic circle.
            local_latitude : The latitude in degree of the point of view on the sphere.
            tropical_inclinaison: The angle in degree of the absolute latitude of tropics.
        """
        self.tropical_inclinaison = tropical_inclinaison
        self.local_latitude = local_latitude
        self.projection_radius = projection_radius

    def project(self, theta):
        """ Return the projected radius of any circle parallel to equator from its latitude angle in degree. """
        return self.projection_radius * tan(pi/4 - deg2rad(theta/2)) / tan(pi/4 - deg2rad(-self.tropical_inclinaison/2))
       
    def project_latitude(self, theta, latitude = None, sign = 1):
        """ Return the height of the center of any circle parallel to a plane tilted by angle from equator. """
        if latitude == None: latitude = self.local_latitude
        return 0.5 * (self.project(latitude - theta) + sign * self.project(latitude + theta))
    
    def angle_arc(self, min_radius, max_radius, h):
        """ Return the angle of the arc of radius radius2 and at a certain height contains in a circle of radius radius2 """
        return rad2deg(arccos((min_radius**2 - max_radius**2 + h**2)/(2*min_radius*h)))
    
    def horizon_radius(self):
        return self.project_latitude(90, sign=-1)

    def capricorn_radius(self):
        return self.project(-self.tropical_inclinaison)